;=======================================;
;Title: DBZ: Assault of the Saiyans 	;
;Nametable load routine rewrite	    	;
;=======================================;
;The purpose of this code is to get rid ;
;of the annoying dakuten/handakuten line;
;once and for all. Previous attempts	;
;were made using less extreme methods of;
;removal, but all failed.		;
;This *should* deal with this problem	;
;once and for all			;
;=======================================;
;Original code can be found starting at ;
;$CA81 in the last bank of PRG-ROM.	;
;=======================================;
;New code can be found starting at $FA06;
;=======================================;

;=======================================;
;6 bytes of RAM are used when dealing	;
;with text. They are as follows.	;
;=======================================;
;$600,x - The number of VRAM writes to	;
;	  make. In the case of text,	;
;	  this is going to always be 2.	;
;					;
;$601,x - Mirroring Select information.	;
;					;
;$602,x - Low byte of the VRAM address	;
;	  to write to.			;
;					;
;$603,x - High byte of the VRAM address	;
;	  to write to.			;
;					;
;$604,x - The dakuten/handakuten data.	;
;					;
;$605,x - The 'base' character data.	;
;					;
;$606,x - Used to terminate the VRAM	;
;	  writes.			;
;=======================================;



.org $fa06
.mem 8
.index 8

code_start:
	lda $2002		;Vblank.
	ldx #$00		;Load X with 0 because we want to start at the beginning of
				;the data to be transfered into VRAM.

load_y:
	ldy $0600,x		;Load Y with the number of vram writes to make, and execute
	beq end_of_routine	;the routine as long as Y doesn't equal 0.

ppu_set_up:
	lda $0601,x
	sta $8009		;Set up the Mirroring/Page Select.

	and #$04
	sta $2000		;Set PPU to Auto-Increment by 32.

vram_set_up:
	lda $0603,x		;Set up the PPU with the address in VRAM we want to
	sta $2006		;write to.
	lda $0602,x
	sta $2006

test_text:
	lda $06ff		;This byte of memory is set to 01 by the text read routine,
	cmp #$01		;and is used here to determine if we branch to the text specific
	bne pre_vram		;code. If 1 then yes, no otherwise.

ye_ol_check:
	lda $0606,x		;Load the byte that determines termination and see if it's
	cmp #$00		;zero or not. This is necessary for when the A button is
				;pressed during a text display. When the 6 characters worth of
				;data is loaded into RAM $0606,x will be overwrote by the
				;number of VRAM writes IF it the current letter isn't the
				;last of the characters to load.
	bne ppu_text_prep

x_check:
	cpx $06fe		;If we're not on the last character to be loaded we don't want the
	bne ppu_text_prep	;test bytes to be reset.

text_reset:
	lda #$00		;If we're on the last character to display, reset the test
	sta $06ff		;bytes.
	sta $06fe

ppu_text_prep:
	dey			;Decrement Y to compensate for only one VRAM write.
	inx			;Increment X so we retrieve data from $0605,x (the base
				;character) to display INSTEAD of $0604,x (the dakuten/
				;handakuten character).

pre_vram:
	tya			;Transfer the value in Y.
	lsr			;Shift it right.
	bcs vram_access_2	;If the least-significant bit is 1
				;only write to VRAM once this run through (the routine).
				;If the least-significant bit is 0
				;write to VRAM twice this run through.

vram_access_1:
	lda $0604,x		;This will read in the dakuten/handakuten information from RAM,
	sta $2007		;and load it into the nametable.
	inx			;Increment X so the base character will be read next.
	dey

vram_access_2:
	lda $0604,x		;This loads the base character into the nametable on the line
	sta $2007		;below the previously loaded dakuten/handakuten.
	inx
	dey
	bne vram_access_1	;if (y != 0) {goto vram_access_1;}
				;This won't happen when dealing with displaying text.

preamble_to_the_end:
	inx			;X is incremented 4 times, so that we read in $0600,(x+4),
	inx			;which will be the termination byte of the current character.
	inx			;If it isn't 00, then we still have characters left to display.
	inx
	bne load_y

end_of_routine:
	lda #$00		;Fail-safe so this routine isn't unintentionally executed again.
	sta $0600
	rts